﻿Type=Class
Version=3.5
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private AppPath As String: AppPath = ""
	Private RemoteUserDir As String
	Private RemoteDir As String
	Private BucketName As String : BucketName=""
	Private BucketLocation As String: BucketLocation="us-east-1"
	Private awsS3Location As String: awsS3Location="s3.amazonaws.com"
	Private FileMimeType As String: FileMimeType = ""
	Private awsLink As String: awsLink="http://" & BucketName & "." & awsS3Location
	'Dim FileMimeType2 As String: FileMimeType2 = "application/x-download"
	Private EOL As String :	EOL = Chr(10)
	Private AccessKey As String, SecretKey As String
	Private debug As Boolean: debug=True
	'Remote/change these later
	AccessKey=""
	SecretKey=""
	Type HeaderType(key As String, value As String)
End Sub
#Region Public Functions
'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize()
	'Dim P As Phone
	'RemoteUserDir = P.GetSettings("android_id")
	'RemoteDir= AppPath & "/" & RemoteUserDir
	'RemoteDir=""
End Sub
'This method is to set the access and secret keys instead of the defaults stored in the class
'Use this to read the keys from a trusted source and set them instead of hard coding them
Public Sub OverrideAccessKeys(AccessKeyOverride As String, SecretKeyOverride As String)
	AccessKey=AccessKeyOverride
	SecretKey=SecretKeyOverride
End Sub
'This method is to set the bucket name, app path and remote directory
'Bucket name is your s3 bucket name.  It is recommended that the name is in all lowercase
'AppPath is an optional string to define a directory to write to.  You can leave this blank to put it in the root dir of the bucket.
'RemoteUserDir by default is set to "P.GetSettings("android_id")" which is a unique string per device.  You can set this blank.
'Set them both to blank to write to root of bucket
Public Sub OverrideBucketInfo(BucketNameOverride As String, AppPathOverride As String, RemoteUserDirOverride As String)
	If BucketNameOverride<>"" Then
		BucketName=BucketNameOverride
		awsLink="http://" & BucketName & "." & awsS3Location
	End If
	AppPath=AppPathOverride
	RemoteUserDir=RemoteUserDirOverride
	If AppPath<>"" AND RemoteUserDir<>"" Then
		RemoteDir= AppPath & "/" & RemoteUserDir
	End If
	If AppPath<>"" AND RemoteUserDir="" Then
		RemoteDir = AppPath 
	End If
	If AppPath="" AND RemoteUserDir<>"" Then
		RemoteDir=RemoteUserDir
	End If
	If AppPath="" AND RemoteUserDir="" Then
		RemoteDir=""
	End If
End Sub
'This method performs a PUT request to Amazon S3 service
'Filedir is the path to the local file, Filename is the file name
'InMap is a set of custom request headers, such as permissions.  Refer to s3 docs on amazon site for more options
'Request Name is the name of the job (for processing a successful return or failure).  Request Target is the name of the module where to send the response for processing.
'BucketOverride is for setting the bucket name.  It's optional and can be set to "" to use already set bucket name
'FileMimeTypeOverride is for setting the file mime type.  It's optional and can be set to "" to use already set File Mime Type
Public Sub AwsPUT(filedir As String, filename As String,RequestName As String, RequestTarget As Object,inMap As Map, bucketOverride As String,FileMimeTypeOverride As String)
	Dim filedata() As Byte
	Dim CurrDate As Long, formattedDateTime As String, formattedDate As String, iso8601date As String
	Dim job1 As HttpJob
	Dim HeaderMap As Map
	Dim HashedPayLoad As String
	DateTime.SetTimeZone(0)
	Dim AWSAuth As String
	Dim HTTPMethod As String, AuthString As String, CurrentBucket As String, CurrentFileMimeType As String
	If filedir<>"" Then
		RemoteUserDir=filedir
	Else
		RemoteDir= AppPath & "/" & RemoteUserDir
	End If
	If bucketOverride="" Then
		CurrentBucket=BucketName
	Else
		CurrentBucket=bucketOverride
	End If
	awsLink="http://" & BucketName & "." & awsS3Location
	If FileMimeTypeOverride="" Then
		CurrentFileMimeType=FileMimeType
	Else
		CurrentFileMimeType=FileMimeTypeOverride
	End If
	'Set Date formats
	CurrDate=DateTime.Now
	formattedDateTime=GenerateCurrDateTime(CurrDate)
	formattedDate=GenerateCurrDate(CurrDate)
	iso8601date=GetISO8601Date(CurrDate)
	
	'Generate File hash and load into byte array
	filedata=ReadFileData(filedir,filename)
	HashedPayLoad=FileHash(filedata)
	
	'Set HTTP Method
	HTTPMethod="PUT"
	
	'Setup Header Strings
	HeaderMap=DefaultPutHeaders(CurrentBucket,HashedPayLoad,iso8601date,CurrentFileMimeType)
	HeaderMap=AddToMap(HeaderMap,inMap)
	HeaderMap=SortMap(HeaderMap)
	
	'Create Auth String
	AuthString=Authenticate(filedir,filename,HTTPMethod,formattedDateTime,formattedDate,filedata,iso8601date,inMap,HeaderMap,CurrentFileMimeType,False)
	AWSAuth="AWS " & AccessKey & ":" & AuthString
	
	If debug=True Then
		Log("------Auth String------")
		Log (AWSAuth)
		Log("------End Auth String--")
	End If
	
	'Send to httpjob
	job1.Initialize(RequestName,RequestTarget)
	job1.PutBytes(awsLink & "/" & RemoteDir & "/" & filename,filedata)
	For x=HeaderMap.Size-1 To 0 Step -1
		If HeaderMap.GetKeyAt(x)<>"Host" Then
			job1.GetRequest.SetHeader(HeaderMap.GetKeyAt(x),HeaderMap.Get(HeaderMap.GetKeyAt(x)))
		End If
	Next
	job1.GetRequest.SetHeader("Authorization",AWSAuth)
End Sub
'This method performs a GET request to Amazon S3 service
'Filedir is the path to the REMOTE file, Filename is the REMOTE file name
'InMap is a set of custom request headers, such as permissions.  Refer to s3 docs on amazon site for more options.  You can leave this empty for this method.
'LocalInDir sets where you want the file to be placed.  LocalInFileName is what you want to call it.  Setting it to "" will keep the same file name as the remote file name.
'BucketOverride is for setting the bucket name.  It's optional and can be set to "" to use already set bucket name
Public Sub AwsGET(filedir As String, filename As String,RequestName As String, RequestTarget As Object,LocalInDir As String, LocalInFileName As String,inMap As Map, bucketOverride As String)
	Dim filedata() As Byte
	Dim CurrDate As Long, formattedDateTime As String, formattedDate As String, iso8601date As String
	Dim job1 As HttpJob
	DateTime.SetTimeZone(0)
	Dim AWSAuth As String
	Dim HeaderMap As Map
	Dim HTTPMethod As String, AuthString As String, CurrentBucket As String,CurrentFileMimeType As String, currentDir As String
	CurrentFileMimeType=""
	'Set up inputs
	If filedir<>"" Then
		currentDir=filedir
	Else
		currentDir=RemoteDir
	End If
	If LocalInFileName="" Then
		LocalInFileName=filename
	End If
	If bucketOverride="" Then
		CurrentBucket=BucketName
	Else
		CurrentBucket=bucketOverride
	End If
	awsLink="http://" & BucketName & "." & awsS3Location
	
	'Set Date formats
	CurrDate=DateTime.Now
	formattedDateTime=GenerateCurrDateTime(CurrDate)
	formattedDate=GenerateCurrDate(CurrDate)
	iso8601date=GetISO8601Date(CurrDate)
	
	'Set HTTP Method
	HTTPMethod="GET"
	
	'Setup Header Strings
	HeaderMap=DefaultGetHeaders(CurrentBucket,iso8601date)
	HeaderMap=AddToMap(HeaderMap,inMap)
	HeaderMap=SortMap(HeaderMap)
	
	'Create Auth String
	AuthString=Authenticate(filedir,filename,HTTPMethod,formattedDateTime,formattedDate,filedata,iso8601date,inMap,HeaderMap,CurrentFileMimeType,False)
	AWSAuth="AWS " & AccessKey & ":" & AuthString
	
	If debug=True Then
		Log("------Auth String------")
		Log (AWSAuth)
		Log("------End Auth String--")
	End If
	
	'Send to httpjob
	job1.Initialize(RequestName,RequestTarget)
	job1.SetDownloadsettings(LocalInDir, LocalInFileName)
	If currentDir<>"" Then
		job1.Download(awsLink & "/" & currentDir & "/" & filename)
	Else
		job1.Download(awsLink & "/" & filename)
	End If
	For x=HeaderMap.Size-1 To 0 Step -1
		If HeaderMap.GetKeyAt(x)<>"Host" Then
			job1.GetRequest.SetHeader(HeaderMap.GetKeyAt(x),HeaderMap.Get(HeaderMap.GetKeyAt(x)))
		End If
	Next
	job1.GetRequest.SetHeader("Authorization",AWSAuth)
	
End Sub
'This method performs a DELETE request to Amazon S3 service
'Filedir is the path to the REMOTE file, Filename is the REMOTE file name
'InMap is a set of custom request headers, such as permissions.  Refer to s3 docs on amazon site for more options.  You can leave this empty for this method.
'BucketOverride is for setting the bucket name.  It's optional and can be set to "" to use already set bucket name
Public Sub AwsDELETE(filedir As String, filename As String,RequestName As String, RequestTarget As Object,inMap As Map, bucketOverride As String)
	Dim filedata() As Byte
	Dim CurrDate As Long, formattedDateTime As String, formattedDate As String, iso8601date As String
	Dim job1 As HttpJob
	DateTime.SetTimeZone(0)
	Dim AWSAuth As String
	Dim HeaderMap As Map
	Dim HTTPMethod As String, AuthString As String, CurrentBucket As String,CurrentFileMimeType As String,currentDir As String
	CurrentFileMimeType=""
	'Set up inputs
	If filedir<>"" Then
		currentDir=filedir
	Else
		currentDir=RemoteDir
	End If
	If bucketOverride="" Then
		CurrentBucket=BucketName
	Else
		CurrentBucket=bucketOverride
	End If
	awsLink="http://" & BucketName & "." & awsS3Location
	'Set Date formats
	CurrDate=DateTime.Now
	formattedDateTime=GenerateCurrDateTime(CurrDate)
	formattedDate=GenerateCurrDate(CurrDate)
	iso8601date=GetISO8601Date(CurrDate)
	
	'Set HTTP Method
	HTTPMethod="DELETE"
	
	'Setup Header Strings
	HeaderMap=DefaultGetHeaders(CurrentBucket,iso8601date)
	HeaderMap=AddToMap(HeaderMap,inMap)
	HeaderMap=SortMap(HeaderMap)
	
	'Create Auth String
	AuthString=Authenticate(filedir,filename,HTTPMethod,formattedDateTime,formattedDate,filedata,iso8601date,inMap,HeaderMap,CurrentFileMimeType,False)
	AWSAuth="AWS " & AccessKey & ":" & AuthString
	
	If debug=True Then
		Log("------Auth String------")
		Log (AWSAuth)
		Log("------End Auth String--")
	End If
	
	'Send to httpjob
	job1.Initialize(RequestName,RequestTarget)
	If currentDir<>"" Then
		job1.DeleteRequest(awsLink & "/" & currentDir & "/" & filename)
	Else
		job1.DeleteRequest(awsLink & "/" & filename)
	End If
	For x=HeaderMap.Size-1 To 0 Step -1
		If HeaderMap.GetKeyAt(x)<>"Host" Then
			job1.GetRequest.SetHeader(HeaderMap.GetKeyAt(x),HeaderMap.Get(HeaderMap.GetKeyAt(x)))
		End If
	Next
	job1.GetRequest.SetHeader("Authorization",AWSAuth)
End Sub
'This method performs a LIST (GET) request to Amazon S3 service
'RemoteFileDir is the path to the REMOTE file, Filename is the REMOTE file name - Do not end with a slash
'InMap is a set of custom request headers, such as permissions.  Refer to s3 docs on amazon site for more options.  You can leave this empty for this method.
'LocalInDir sets where you want the file to be placed.  LocalInFileName is what you want to call it.  Setting it to "" will keep the same file name as the remote file name.
'BucketOverride is for setting the bucket name.  It's optional and can be set to "" to use already set bucket name
'FoldersOnly is a boolean for querying for "folders" (as they exist in s3) to be returned.
Public Sub AwsLIST(RemoteFileDir As String,RequestName As String, RequestTarget As Object,inMap As Map, bucketOverride As String,FoldersOnly As Boolean)
	Dim filedata() As Byte
	Dim CurrDate As Long, formattedDateTime As String, formattedDate As String, iso8601date As String
	Dim job1 As HttpJob
	DateTime.SetTimeZone(0)
	Dim AWSAuth As String
	Dim HeaderMap As Map
	Dim HTTPMethod As String, AuthString As String, CurrentBucket As String,CurrentFileMimeType As String, currentDir As String
	'Set up inputs
	CurrentFileMimeType="text/plain"
	inMap.Put("Content-Type",CurrentFileMimeType)
	
	If RemoteFileDir<>"" Then
		currentDir=RemoteFileDir
	Else
		currentDir=RemoteDir
	End If
	If bucketOverride="" Then
		CurrentBucket=BucketName
	Else
		CurrentBucket=bucketOverride
	End If
	awsLink="http://" & BucketName & "." & awsS3Location
	'Set Date formats
	CurrDate=DateTime.Now
	formattedDateTime=GenerateCurrDateTime(CurrDate)
	formattedDate=GenerateCurrDate(CurrDate)
	iso8601date=GetISO8601Date(CurrDate)
	
	'Set HTTP Method
	HTTPMethod="GET"
	
	'Setup Header Strings
	HeaderMap=DefaultGetHeaders(CurrentBucket,iso8601date)
	HeaderMap=AddToMap(HeaderMap,inMap)
	HeaderMap=SortMap(HeaderMap)
	
	'Create Auth String
	AuthString=Authenticate(currentDir,"",HTTPMethod,formattedDateTime,formattedDate,filedata,iso8601date,inMap,HeaderMap,CurrentFileMimeType,True)
	AWSAuth="AWS " & AccessKey & ":" & AuthString
	
	If debug=True Then
		Log("------Auth String------")
		Log (AWSAuth)
		Log("------End Auth String--")
	End If
	
	'Send to httpjob
	job1.Initialize(RequestName,RequestTarget)
	If currentDir<>"" Then
		If FoldersOnly=False Then
			job1.Download(awsLink & "/?prefix=" & currentDir)
		Else
			job1.Download(awsLink & "/?prefix=" & currentDir & "&delimiter=/")
		End If
	Else
		If FoldersOnly=False Then
			job1.Download(awsLink & "/")
		Else
			job1.Download(awsLink & "/"& "?delimiter=/")
		End If
	End If
	For x=HeaderMap.Size-1 To 0 Step -1
		If HeaderMap.GetKeyAt(x)<>"Host" Then
			job1.GetRequest.SetHeader(HeaderMap.GetKeyAt(x),HeaderMap.Get(HeaderMap.GetKeyAt(x)))
		End If
	Next
	job1.GetRequest.SetHeader("Authorization",AWSAuth)
	
End Sub
#End Region
#Region Setup Headers
'Sets up default put headers for put method
Private Sub DefaultPutHeaders(CurrentBucket As String,HashedPayLoad As String,iso8601date As String,CurrentFileMimeType As String) As Map
	Dim tempmap As Map
	tempmap.Initialize
	tempmap.Put("Host",CurrentBucket & "." & awsS3Location)
	tempmap.Put("x-amz-content-sha256",HashedPayLoad)
	tempmap.Put("Date",iso8601date)
	tempmap.Put("Content-Type",CurrentFileMimeType)
	Return tempmap
End Sub
Private Sub DefaultGetHeaders(CurrentBucket As String,iso8601date As String) As Map
	Dim tempmap As Map
	tempmap.Initialize
	tempmap.Put("Host",CurrentBucket & "." & awsS3Location)
	tempmap.Put("Date",iso8601date)
	Return tempmap
End Sub
Private Sub CreateAuthHeader(Authstring As String,formatteddate As String, HeaderMap As Map) As String
	Dim TempString As String
	Dim b() As Byte
	Dim s As StringUtils
	Dim bc As ByteConverter
	b=s.DecodeBase64(Authstring)
	
	TempString="AWS4-HMAC-SHA256 Credential=" & AccessKey & "/" & formatteddate & "/" & BucketLocation & "/s3/aws4_request,"
	TempString = TempString & " SignedHeaders=" & createSignedHeaders(HeaderMap) & ","
	TempString= TempString & " Signature=" & bc.HexFromBytes(b)
	Return TempString
End Sub
#End Region
#Region Authentication Methods
Private Sub Authenticate(filedir As String,filename As String,HTTPMethod As String,formattedDateTime As String,formattedDate As String,filedata() As Byte,iso8601date As String,inMap As Map, HeaderMap As Map, CurrentFileMimeType As String,listtype As Boolean) As String
Dim StringtoSign As String

StringtoSign=createStringToSign(HTTPMethod, iso8601date,formattedDateTime,filename,HeaderMap,CurrentFileMimeType,filedir,listtype)
Return HMACSHA1Encode(SecretKey, StringtoSign)
End Sub

Private Sub createStringToSign(HTTPMethod As String, iso8601date As String,formattedDate As String,filename As String,HeaderMap As Map,CurrentFileMimeType As String,filedir As String,listtype As Boolean) As String
	Dim StringToSign As String
	StringToSign=HTTPMethod & EOL
	StringToSign = StringToSign & EOL
	StringToSign = StringToSign & CurrentFileMimeType & EOL
	StringToSign = StringToSign & iso8601date & EOL
	StringToSign = StringToSign & CreateCanonicalHeaders(HeaderMap)
	If listtype=True Then
		StringToSign = StringToSign & "/" & BucketName & "/"
	Else
		If filedir<>"" Then
			StringToSign = StringToSign & "/" & BucketName & "/" & filedir & "/" & filename
		Else
			StringToSign = StringToSign & "/" & BucketName & "/" & filename
		End If
	End If
	If debug=True Then
		Log ("----startof stringtosign-----")
		Log(StringToSign)
		Log ("------endof stringtosign-----")
	End If
	Return StringToSign
End Sub

Private Sub createStringToSignold(CanonicalRequest As String,iso8601date As String,formattedDate As String) As String
	Dim StringToSign As String
	StringToSign="AWS4-HMAC-SHA256" & EOL
	StringToSign = StringToSign & iso8601date & EOL
	StringToSign = StringToSign & formattedDate & "/" & BucketLocation & "/s3/aws4_request" & EOL
	StringToSign = StringToSign & textHash(CanonicalRequest)
	If debug=True Then
		Log ("----startof stringtosign-----")
		Log(StringToSign)
		Log ("------endof stringtosign-----")
	End If
	Return StringToSign
End Sub
Private Sub CreateCanonicalRequestPut(HTTPMethod As String, HashedPayLoad As String,iso8601date As String, filename As String,InMap As Map, QueryMap As Map,formatteddatetime As String,HeaderMap As Map) As String
	Dim RequestString As String, canonicalheaders As String, signedheaders As String
	Dim QueryString As String
	
	QueryString=createCanonicalQueryString(QueryMap)
	canonicalheaders=CreateCanonicalHeaders(HeaderMap)
	signedheaders=createSignedHeaders(HeaderMap)
	
	RequestString= HTTPMethod & EOL
	RequestString=RequestString &  "/" & RemoteDir  & "/" & filename & EOL
	RequestString=RequestString &  QueryString & EOL'"http://" & awsS3Location & "/" & BucketName ' -- check if this is necessary & "?" & QueryString & EOL
	RequestString=RequestString &  canonicalheaders & EOL
	RequestString=RequestString &  signedheaders & EOL
	RequestString=RequestString &  HashedPayLoad
	'Log(RequestString)
	Return RequestString
End Sub
'Inputs a headermap
'Outputs a uri encoded string for a query
Private Sub createCanonicalQueryString(headermap As Map) As String
	Dim querystring As String, x As Int
	querystring=""
	If headermap.IsInitialized Then
		For x=0 To headermap.Size-1
			querystring=querystring & EncodeURI(headermap.GetKeyAt(x)) & "=" & EncodeURI(headermap.Get(headermap.GetKeyAt(x)))
			If x<>headermap.Size-1 Then
				querystring=querystring & "&"
			End If
		Next
	End If
	Return querystring
End Sub

'Inputs a headermap
'Outputs all headermap variables into a string for the string to sign
Private Sub CreateCanonicalHeadersold(HeaderMap As Map) As String
	Dim headerstring As String,x As Int, mapstring As String, Mapvalue As String
	headerstring=""
	For x=0 To HeaderMap.Size-1
		mapstring=HeaderMap.GetKeyAt(x)
		Mapvalue=HeaderMap.Get(HeaderMap.GetKeyAt(x))
		headerstring = headerstring & mapstring.ToLowerCase & ":" & Mapvalue.Trim & EOL
		
	Next
	If debug=1 Then
		Log ("--------headerstring----------")
		Log(headerstring)
		Log ("-------end headerstring-------")
	End If
	Return headerstring
End Sub
'Inputs a HeaderMap
'Ignores Date, Content-Type, Host and outputs custom headers into a string for the string to sign
Private Sub CreateCanonicalHeaders(HeaderMap As Map) As String
	Dim headerstring As String,x As Int, mapstring As String, Mapvalue As String
	headerstring=""
	For x=0 To HeaderMap.Size-1
		
		mapstring=HeaderMap.GetKeyAt(x)
		If mapstring<>"Date" AND mapstring<>"Content-Type" AND mapstring<>"Host" Then
			Mapvalue=HeaderMap.Get(HeaderMap.GetKeyAt(x))
			headerstring = headerstring & mapstring.ToLowerCase & ":" & Mapvalue.Trim & EOL
		End If
	Next
	If debug=True Then
		Log ("--------headerstring----------")
		Log(headerstring)
		Log ("-------end headerstring-------")
	End If
	Return headerstring
End Sub
Private Sub createSignedHeaders(HeaderMap As Map) As String
	Dim headerstring As String,x As Int, mapstring As String
	headerstring = ""
	For x=0 To HeaderMap.Size-1
		mapstring=HeaderMap.GetKeyAt(x)
		headerstring = headerstring & mapstring.ToLowerCase
		If x<>HeaderMap.Size-1 Then
			headerstring=headerstring & ";"
		End If
	Next
	'headerstring=headerstring.SubString2(1,headerstring.Length)
	If debug=True Then
		Log ("--------headerstring----------")
		Log(headerstring)
		Log ("-------end headerstring-------")
	End If
	Return headerstring
End Sub
'Old signing key method
Private Sub generateSigningKeyold(DateString As String) As String
	Log("HMAC-SHA265 Key=" & "AWS4" & SecretKey & " String=" & DateString)
	Log("HMAC-SHA265 Key=" & HMACSHA256Encode("AWS4" & SecretKey,DateString) & " String=" & BucketLocation)
	Log("HMAC-SHA265 Key=" & HMACSHA256Encode2(HMACSHA256Encode("AWS4" & SecretKey,DateString),BucketLocation) & " String=" & "s3")
	Log("HMAC-SHA265 Key=" & HMACSHA256Encode2(HMACSHA256Encode2(HMACSHA256Encode("AWS4" & SecretKey,DateString),BucketLocation),"s3") & " String=" & "aws4_request")
	Return HMACSHA256Encode2(HMACSHA256Encode2(HMACSHA256Encode2(HMACSHA256Encode("AWS4" & SecretKey,DateString),BucketLocation),"s3"),"aws4_request")
End Sub
#End Region
#Region File Methods
'Reads a file, returns as byte array of it
Private Sub ReadFileData(filedir As String,filename As String) As Byte()
	Dim FileData() As Byte
	
	Dim In As InputStream
	In = File.OpenInput(filedir,filename)
    Dim out As OutputStream
    out.InitializeToBytesArray(1000)
    File.Copy2(In, out) 
    Dim data() As Byte
    data = out.ToBytesArray
	Return data
End Sub
#End Region
#Region Encryption and hash methods
'Uses the encryption library to encrypt a key (string) and input (string) using SHA1 and UTF8 characters
'Returns a EncodeBase64 string
Private Sub HMACSHA1Encode(key As String, input As String) As String
   Dim m As Mac
   Dim k As KeyGenerator
   Dim Base64String As StringUtils
   k.Initialize("HMACSHA1")
   k.KeyFromBytes(key.GetBytes("UTF8"))
   m.Initialise("HMACSHA1", k.key)
   m.Update(input.GetBytes("UTF8"))
   Dim b() As Byte
   b = m.Sign
   Dim bc As ByteConverter
   'Log(bc.HexFromBytes(b))
   Return Base64String.EncodeBase64(b)
 
End Sub
'Uses the encryption library to encrypt a key (string) and input (string) using SHA256 and UTF8 characters
'Returns a EncodeBase64 string
Private Sub HMACSHA256Encode(key As String, input As String) As String
   Dim m As Mac
   Dim k As KeyGenerator
   Dim Base64String As StringUtils
   Dim tempstring As String
   k.Initialize("HMACSHA256")
   k.KeyFromBytes(key.GetBytes("UTF8"))
   m.Initialise("HMACSHA256", k.key)
   m.Update(input.GetBytes("UTF8"))
   Dim b() As Byte
   b = m.Sign
   Dim bc As ByteConverter
   'Log(bc.HexFromBytes(b))
   tempstring=bc.HexFromBytes(b)
   tempstring=bc.CharsFromBytes(b)
   Return Base64String.EncodeBase64(b)
 
End Sub
'Uses the encryption library to encrypt a key (string) and input (Encoded base 64 string) using SHA256 and UTF8 characters
'Returns a EncodeBase64 string
Private Sub HMACSHA256Encode3(key As String, input As String) As String
   Dim m As Mac
   Dim k As KeyGenerator
   Dim Base64String As StringUtils
   k.Initialize("HMACSHA256")
   k.KeyFromBytes(key.GetBytes("UTF8"))
   m.Initialise("HMACSHA256", k.key)
   m.Update(Base64String.DecodeBase64(input))
   Dim b() As Byte
   b = m.Sign
   Return Base64String.EncodeBase64(b)
End Sub
'Uses the encryption library to encrypt a key (Encoded base 64 string) and input (string) using SHA256 and UTF8 characters
'Returns a EncodeBase64 string
Private Sub HMACSHA256Encode2(key As String, input As String) As String
   Dim m As Mac
   Dim k As KeyGenerator
   Dim Base64String As StringUtils
   k.Initialize("HMACSHA256")
   k.KeyFromBytes(Base64String.DecodeBase64(key))
   m.Initialise("HMACSHA256", k.key)
   m.Update(input.GetBytes("UTF8"))
   Dim b() As Byte
   b = m.Sign
   Return Base64String.EncodeBase64(b)
End Sub
'Reads a file, returns the SHA-256 hash of it in a hex string(lowercase)
Private Sub FileHash (filedata() As Byte) As String
'input is file location
	Dim bc As ByteConverter 
	Dim b64 As Base64
	Dim md As MessageDigest
	Dim encrypted() As Byte
	Dim tempstring As String
	'data = bc.StringToBytes("Hello world", "utf-8")
	encrypted = md.GetMessageDigest(filedata, "SHA-256")
    tempstring= bc.HexFromBytes(encrypted)
	Return tempstring.ToLowerCase
End Sub
'Reads a string, returns the SHA-256 hash of it in a hex string(lowercase)
Private Sub textHash (textdata As String) As String
	Dim tempString As String
	Dim bc As ByteConverter 
	Dim md As MessageDigest

	Dim encrypted() As Byte


	encrypted = md.GetMessageDigest(textdata.GetBytes("UTF8"),"SHA-256")
	tempString= bc.HexFromBytes(encrypted)
	
	Return tempString.ToLowerCase

End Sub
#End Region
#Region Date Time methods
'Inputs a time in android number format
'Returns a formatted date string
Private Sub GenerateCurrDate(currTime As Long) As String
	Dim CurrDay As String
	Dim CurrMonth As String
	Dim CurrMonthNum As String
	Dim CurrYear As String
	Dim CurrDayNum As String
	Dim CurrHour As String
	Dim CurrMin As String
	Dim CurrSecond As String
	Dim TZ As String
	CurrDay=DateUtils.GetDayOfWeekName(currTime)
	CurrMonth=DateUtils.GetMonthName(currTime)
	CurrMonthNum=DateTime.GetMonth(currTime)
	CurrYear=DateTime.GetYear(currTime)
	CurrDayNum=DateTime.GetDayOfMonth(currTime)
	CurrHour=DateTime.GetHour(currTime)
	CurrMin=DateTime.GetMinute(currTime)
	CurrSecond=DateTime.GetSecond(currTime)
	If debug=True Then
		Log (CurrYear & CurrMonthNum & CurrDayNum)
	End If
	DateTime.DateFormat="yyyyMMdd"
	Return DateTime.Date(currTime)
	'Return CurrYear & CurrMonthNum & CurrDayNum
End Sub
'Inputs a time in android number format
'Returns a formatted date/time string
Private Sub GenerateCurrDateTime(currTime As Long) As String
	Dim CurrDay As String
	Dim CurrMonth As String
	Dim CurrYear As String
	Dim CurrDayNum As String
	Dim CurrHour As String
	Dim CurrMin As String
	Dim CurrSecond As String
	Dim TZ As String
	CurrDay=DateUtils.GetDayOfWeekName(currTime)
	CurrMonth=DateUtils.GetMonthName(currTime)
	CurrYear=DateTime.GetYear(currTime)
	CurrDayNum=DateTime.GetDayOfMonth(currTime)
	CurrHour=DateTime.GetHour(currTime)
	CurrMin=DateTime.GetMinute(currTime)
	CurrSecond=DateTime.GetSecond(currTime)
	If debug=True Then
		Log (CurrDay & ", " & CurrDayNum & " " & CurrMonth & " " & CurrYear & " " & CurrHour & ":" & CurrMin & ":" & CurrSecond & " +0000")
	End If
	Return CurrDay & ", " & CurrDayNum & " " & CurrMonth & " " & CurrYear & " " & NumberFormat(CurrHour,2,0) & ":" & NumberFormat(CurrMin,2,0) & ":" & NumberFormat(CurrSecond,2,0) & " +0000"
	
End Sub
Private Sub GetISO8601Date(currTime As Long) As String
	Dim StrResult As String = ""
	Dim CurrDay As String
	Dim CurrMonth As String
	Dim CurrMonthNum As String
	Dim CurrYear As String
	Dim CurrDayNum As String
	Dim CurrHour As String
	Dim CurrMin As String
	Dim CurrSecond As String
	Dim TZ As String
	CurrDay=DateUtils.GetDayOfWeekName(currTime)
	CurrMonth=DateUtils.GetMonthName(currTime)
	CurrMonthNum=DateTime.GetMonth(currTime)
	CurrYear=DateTime.GetYear(currTime)
	CurrDayNum=DateTime.GetDayOfMonth(currTime)
	CurrHour=DateTime.GetHour(currTime)
	CurrMin=DateTime.GetMinute(currTime)
	CurrSecond=DateTime.GetSecond(currTime)
	Dim StrISO8601Date As String = NumberFormat2(CurrYear,4,0,0,False) & "" & NumberFormat2(CurrMonthNum,2,0,0,False) & "" & NumberFormat2(CurrDayNum,2,0,0,False) & "T" & NumberFormat2(CurrHour,2,0,0,False) & "" & NumberFormat2(CurrMin,2,0,0,False) & "" & NumberFormat2(CurrSecond,2,0,0,False) & "Z"'"." & NumberFormat2 (Millisecond,3,0,0,False)
	StrResult = StrISO8601Date
	Return StrResult
End Sub
#End Region
#Region utility methods
'Inputs a currentMap, an input map and returns a merged map
Private Sub AddToMap(currentMap As Map, inmap As Map) As Map
	If inmap.IsInitialized Then
		For x = 0 To inmap.Size-1
			currentMap.Put(inmap.GetKeyAt(x),inmap.get(inmap.GetKeyAt(x)))
		Next
	End If
	Return currentMap
End Sub
'Inputs a filename
'Outputs a path and filename encoded as a uri string
Private Sub CreateCanonicalURI(filename As String) As String
	Log (EncodeURI("/" & RemoteDir & "/" & filename))
	Return EncodeURI("/" & RemoteDir & "/" & filename)
End Sub 
'Uses reflection to encode a string into a URI encoded string
Private Sub EncodeURI(input As String) As String
	Dim Obj1 As Reflector
	Dim args(1) As Object
	Dim types(1) As String
	args(0) = input
	types(0) = "java.lang.String"
	Return Obj1.RunStaticMethod("android.net.Uri", "encode", args, types)
	'str = args(0) & CRLF & Obj1.RunStaticMethod("android.net.Uri", "encode", args, types)
End Sub
'Sorts a map by alphabetical order and returns that map back
Private Sub SortMap(m As Map) As Map
  	Dim list1 As List, x As Int, data As HeaderType
	list1.Initialize
  	For x = 0 To m.Size-1
 	  	Dim data As HeaderType
		data.key=m.GetKeyAt(x)
		data.value=m.Get(m.GetKeyAt(x))
		list1.Add(data)
	Next
    list1.SortTypeCaseInsensitive("key",True)
    Dim m As Map
	m.Initialize
    For x=0 To list1.Size-1
		data=list1.Get(x)
   		m.Put(data.key,data.value)
    Next
	Return m
End Sub
#End Region







